/*

   Edit toolbar dialog

   Re-arrange buttons in selected toolbar

*/


function Groowe_OnLoad()
{
   try {
      searchEngine = window.arguments[0];
      document.getElementById("Groowe-EditToolbar").attributes.getNamedItem("title").nodeValue = "Edit " + searchEngine + " toolbar";
      
      var xmlContent = Groowe_GetPluginContent(searchEngine);
      if (xmlContent == "") {
         xmlContent = Groowe_GetContents("chrome://groowebar/content/data/engines/" + searchEngine + ".xml"); 
      }      
      
      var parser = new DOMParser();
      var dom = parser.parseFromString(xmlContent, "text/xml");
         
      var nodeList = dom.documentElement.getElementsByTagName("Items");
      var itemsNode = nodeList.item(0);
      var order = "";      
   
      // read native order      
      for (var i = 0; i < itemsNode.childNodes.length; i++) {
         var node = itemsNode.childNodes.item(i);
         if (node.nodeName == "Item") {
            if (node.attributes.getNamedItem("ID") != null) {
               var id = node.attributes.getNamedItem("ID").nodeValue;
               order += (order != "") ? ";" + id : id;
            }
         }
      }   
      var allButtons = order;
      
      // read order xml element
      if (dom.documentElement.getElementsByTagName("Order").length != 0) {
         var tmp = dom.documentElement.getElementsByTagName("Order").item(0).textContent;
         if (tmp != "") {
            order = tmp;
         }
      }
      
      // read customized order setting
      var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
      var branch = prefs.getBranch("groowebar.order.");
      try{ order = branch.getCharPref(searchEngine);} catch(e) {}
   
      Groowe_AddButtons(document.getElementById("Groowe-Btn-Selected"), order, "", itemsNode);
      Groowe_AddButtons(document.getElementById("Groowe-Btn-All"), allButtons, order, itemsNode);
   }
   catch (e) {
      alert(e);
   }
}


function Groowe_AddButtons(container, order, ignore, itemsNode)
{
   try {
      var orderArray = order.split(";");
      var ignoreArray = ignore.split(";");
      
      for (var j = 0; j < orderArray.length; j++) {
         var id = orderArray[j];

         // ignore already selected buttons         
         var process = true;
         for (var i=0; i < ignoreArray.length; i++) {
            if (ignoreArray[i] == id) {
               process = false;
               break;
            }
         }
         if (!process) {
            continue;
         }
         
         if (id == 0) {
            Groowe_AddSeparator();
         }
         else {
            for (var i = 0; i < itemsNode.childNodes.length; i++) {
               var node = itemsNode.childNodes.item(i);
               if ((node.nodeName == "Item") && (node.attributes.getNamedItem("ID") != null) && (node.attributes.getNamedItem("ID").nodeValue == id)) {   
                  var tempItem = null;
                  tempItem = document.createElement("listitem");
                  tempItem.setAttribute("id", id);
                  
                  var caption = "";
                  if (node.attributes.getNamedItem("Caption")) {
                     caption = node.attributes.getNamedItem("Caption").nodeValue
                  }
                  
                  var icon = "";
                  if (node.attributes.getNamedItem("IconID")) {
                     icon = node.attributes.getNamedItem("IconID").nodeValue
                  }                  
                  
                  tempItem.setAttribute("label", caption);
                  if (icon != "") {
                     tempItem.setAttribute("class", "listitem-iconic");
                     tempItem.setAttribute("image", "chrome://groowebar/skin/icons/" + icon + ".png");         
                  }   
                  container.appendChild(tempItem);      
               }
            }
         }
      } 
      container.selectedIndex = 0;
   }
   catch (e) {
      //alert(e);
   }
}


function Groowe_AddSeparator()
{
   var tempItem = null;
   tempItem = document.createElement("listitem");
   tempItem.setAttribute("class", "listitem-iconic");
   tempItem.setAttribute("image", "chrome://groowebar/skin/separator.png");             
   tempItem.setAttribute("id", "0");
   document.getElementById("Groowe-Btn-Selected").appendChild(tempItem);      
   document.getElementById("Groowe-Btn-Selected").selectedItem = tempItem;
}

//
// Move favorite item one place up
//
function Groowe_BtnMoveUp()
{
   try {
      var lb = document.getElementById("Groowe-Btn-Selected");
      var index = lb.selectedIndex;
      
      if (index != 0) {
         var tempItem = lb.childNodes.item(index);         
         lb.removeChild(lb.childNodes.item(index));
         lb.insertBefore(tempItem, lb.childNodes.item(index-1));
         lb.selectedItem = tempItem;
      }
   }
   catch (e) {
      //alert(e);
   }
}



//
// Move favorite item one place down
//
function Groowe_BtnMoveDown()
{
   try {
      var lb = document.getElementById("Groowe-Btn-Selected");
      var index = lb.selectedIndex;

      if (index != lb.childNodes.length-1) {
         var tempItem = lb.childNodes.item(index);         
         lb.removeChild(lb.childNodes.item(index));
         lb.insertBefore(tempItem, lb.childNodes.item(index+1));
         lb.selectedItem = tempItem;
      }
   }
   catch (e) {
      //alert(e);
   }
}

function Groowe_Remove()
{
   try {
      var lbSel = document.getElementById("Groowe-Btn-Selected");
      var lbAll = document.getElementById("Groowe-Btn-All");
      var index = lbSel.selectedIndex;
      
      if (lbSel.childNodes.length == 1) {
         alert("Can't remove all buttons");
         return;
      }
      
      if (index != -1) {
         var tempItem = lbSel.childNodes.item(index);         
         lbSel.removeChild(lbSel.childNodes.item(index));
         if (tempItem.attributes.getNamedItem("id").nodeValue != 0) {
            lbAll.insertBefore(tempItem, lbAll.childNodes.item(lbAll.childNodes.length));
         }   
      }
   }
   catch (e) {
      //alert(e);
   }
}


function Groowe_Add()
{
   try {
      var lbSel = document.getElementById("Groowe-Btn-Selected");
      var lbAll = document.getElementById("Groowe-Btn-All");
      var index = lbAll.selectedIndex;
      
      if (index != -1) {
         var tempItem = lbAll.childNodes.item(index);         
         lbAll.removeChild(lbAll.childNodes.item(index));
         lbSel.insertBefore(tempItem, lbSel.childNodes.item(lbSel.childNodes.length));
         lbSel.selectedItem = tempItem;
      }
   }
   catch (e) {
      //alert(e);
   }
}



function Groowe_SaveOrder()
{
   try {
      var order = "";
      var selButtons = document.getElementById("Groowe-Btn-Selected");
      for (var i = 0; i < selButtons.childNodes.length; i++) {
         if (order != "") order += ";";
         order += selButtons.childNodes.item(i).attributes.getNamedItem("id").nodeValue;
      }
   
      var searchEngine = window.arguments[0];
   
      var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
      var branch = prefs.getBranch("groowebar.order.");
      branch.setCharPref(searchEngine, order); 
   }
   catch (e) {
      //alert(e);
   }
   
}


