/*
   Groowebar v1.6.11
   Groowe Search Toolbar for Firefox

   Author: Milos Mileusnic
   http://www.groowe.com

*/


const groowe_version = "1.6.11";
const groowe_finder = Components.classes['@mozilla.org/embedcomp/rangefind;1'].createInstance().QueryInterface(Components.interfaces.nsIFind);
var groowe_groowe_defaultURL = null;
var Groowe_highlightColors = new Array("#FF0", "#0FF", "#0F0", "#F0F", "orange", "dodgerblue");
var groowe_clearHistoryOnExit = false;
var groowe_HighlightOn = true;
var groowe_HighlightString = "";
var groowe_showCaptions = true;
var groowe_newTab = false;
var groowe_language = "en-US";
var groowe_history = null;
var groowe_autoSelect = true;
var groowe_SearchEngines = null;
var groowe_Categories = null;
var groowe_Homepages = null;
var groowe_Hosts = null;
var groowe_currentQueryField = "";
var groowe_currentHosts = null;
var groowe_currentEngine = "";
var groowe_prevEngine = "";
var groowe_browser = "";
var groowe_forceEngineReload = false;
var groowe_user_agent = "";
var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
var observer_active = true;
var groowe_alert_result = false;
var groowe_new_plugin_category = "";
var groowe_refresh = false;
var groowe_find_in_page_term = "";
var groowe_first_search = true;

//
// On Groowe load
// Add event listener and initialize toolbar
//
function Groowe_Load()
{
   try {
      document.getElementById("Groowe-Toolbar").hidden=false;
      window.getBrowser().addProgressListener(Groowe_Listener, Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
      gBrowser.addEventListener("load", onTabLoad, true);
      Groowe_Init();
   }
   catch (e) {
      Groowe_Error(e);
   }
}


//
// On groowe unload
// Remove event listener
//
function Groowe_UnLoad()
{
   if (groowe_clearHistoryOnExit) {
      Groowe_ClearSearchHistory();
   }
   window.getBrowser().removeProgressListener(Groowe_Listener);
   gBrowser.removeEventListener("load", onTabLoad, true);
}



function Groowe_ReloadOptions()
{
   try {
      // load preferences
      var favorites = null;
      var showHighlight = true;
//      var showHide = true;
      var showFindOnPage = true;
      var showAmazon = false;
      var branch = groowe_prefs.getBranch("groowebar.");
      try {favorites = branch.getCharPref("Favorites");} catch (e) {}
      try {groowe_language = branch.getCharPref("Language");} catch (e) {}
      try {showHighlight = branch.getBoolPref("ShowHighlight");} catch (e) {}
      //try {showHide = branch.getBoolPref("ShowHide");} catch (e) {}
//      try {showAmazon = branch.getBoolPref("ShowAmazon");} catch (e) {}
      try {showFindOnPage = branch.getBoolPref("ShowFindOnPage");} catch (e) {}
      try {groowe_clearHistoryOnExit = branch.getBoolPref("ClearOnExit");} catch (e) {}
      try {groowe_showCaptions = branch.getBoolPref("ShowCaptions");} catch (e) {}
      try {groowe_autoSelect = branch.getBoolPref("AutoSelect");} catch (e) {}
      try {groowe_newTab = branch.getBoolPref("NewTab");} catch (e) {}

      document.getElementById("Groowe-HighlightBtn").hidden = !showHighlight;
      //document.getElementById("Groowe-HideBtn").hidden = !showHide;
      document.getElementById("Groowe-Amazon").hidden = !showAmazon;
      document.getElementById("Groowe-FindOnPageBtn").hidden = !showFindOnPage;
   }
   catch (e) {
      Groowe_Error(e);
   }

}

//
// Initialize Groowebar, load preferences, load default search engine etc.
//
function Groowe_Init()
{
   try {
      // load preferences
      var favorites = null;
      var showHighlight = true;
//      var showHide = true;
      var showFindOnPage = true;
      var showAmazon = false;
      var branch = groowe_prefs.getBranch("groowebar.");
      try {favorites = branch.getCharPref("Favorites");} catch (e) {}
      try {groowe_language = branch.getCharPref("Language");} catch (e) {}
      try {showHighlight = branch.getBoolPref("ShowHighlight");} catch (e) {}
      //try {showHide = branch.getBoolPref("ShowHide");} catch (e) {}
//      try {showAmazon = branch.getBoolPref("ShowAmazon");} catch (e) {}
      try {showFindOnPage = branch.getBoolPref("ShowFindOnPage");} catch (e) {}
      try {groowe_clearHistoryOnExit = branch.getBoolPref("ClearOnExit");} catch (e) {}
      try {groowe_showCaptions = branch.getBoolPref("ShowCaptions");} catch (e) {}
      try {groowe_autoSelect = branch.getBoolPref("AutoSelect");} catch (e) {}
      try {groowe_newTab = branch.getBoolPref("NewTab");} catch (e) {}
      groowe_browser = navigator.vendor;
      groowe_user_agent = navigator.userAgent;

      Groowe_EnumerateEngines();

      groowe_forceEngineReload = true;

      document.getElementById("Groowe-HighlightBtn").hidden = !showHighlight;
      //document.getElementById("Groowe-HideBtn").hidden = !showHide;
      document.getElementById("Groowe-Amazon").hidden = !showAmazon;
      document.getElementById("Groowe-FindOnPageBtn").hidden = !showFindOnPage;

      try {
         if ((favorites != null) && (favorites != "")) {
            groowe_Favorites = favorites.split("||");
         }
      }
      catch (e) {
         groowe_Favorites = groowe_FavoritesEx;
      }

      // filter favorites, remove engines from favorites that don't exists any more
      favorites = "";
      var isFavoritesChanged = false;
      for (var key in groowe_Favorites) {
         var fav = groowe_SearchEngines[groowe_Favorites[key]];
         if (fav != null) {
            favorites += (favorites != "") ? "||" + groowe_Favorites[key] : groowe_Favorites[key];
         }
         else {
            isFavoritesChanged = true;
         }
      }
      if (isFavoritesChanged) {
         branch.setCharPref("Favorites", favorites);
         groowe_Favorites = favorites.split("||");
      }

      groowe_history = new Array();

      // load default search engine (first in favorites)
      Groowe_LoadSearchEngine(groowe_Favorites[0]);

      // load search history
      Groowe_PopulateHistory();

      // populate menu with search engines
      Groowe_ReadEngines();
   }
   catch (e) {
      Groowe_Error(e);
   }
}


//
// Read list of all engines, categories, icons and host
// put values in global arrays
//
function Groowe_EnumerateEngines()
{
   try {
      // initialize global arrays
      groowe_SearchEngines = new Array();
      groowe_Categories = new Array();
      groowe_Hosts = new Array();
      groowe_Homepages = new Array();

      var arr_engines = [];

      // read engines from plugin folder
      var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
      file.append("GrooweFiles");
      try {
         if (file.exists()) {
            var entries = file.directoryEntries;
            while(entries.hasMoreElements()) {
               var entry = entries.getNext();
               entry.QueryInterface(Components.interfaces.nsIFile);
               try {
                  if (entry.leafName.substr(entry.leafName.length-4, 4).toLowerCase() == ".xml") {
                     arr_engines.push(entry.leafName.substr(0, entry.leafName.length-4));
                  }
               }
               catch (e) {
                  Groowe_Error(e);
               }
            }
         }
      }
      catch (e) {
         Groowe_Error(e);
      }


      // read engines from XML
      xmlContent = Groowe_GetContents("chrome://groowebar/content/data/engines.xml");
      var parser = new DOMParser();
      var dom = parser.parseFromString(xmlContent, "text/xml");
      var nodeList = dom.documentElement.getElementsByTagName("Engine");
      for (var i = 0; i < nodeList.length; i++) {
         if (!Groowe_InArray(arr_engines, nodeList.item(i).textContent)) {
            arr_engines.push(nodeList.item(i).textContent);
         }
      }


      // load icons, hosts etc. from engines config files
      for (var i = 0; i < arr_engines.length; i++) {
         try {
            var engineName = arr_engines[i];
            var engineXml = Groowe_GetPluginContent(engineName);
            if (engineXml == "") {
               engineXml = Groowe_GetContents("chrome://groowebar/content/data/engines/" + engineName + ".xml");
            }
            var engineParser = new DOMParser();
            var engineDom = engineParser.parseFromString(engineXml, "text/xml");
            var iconName = (engineDom.documentElement.getElementsByTagName("Icon").item(0).attributes.getNamedItem("IconID") != null) ? engineDom.documentElement.getElementsByTagName("Icon").item(0).attributes.getNamedItem("IconID").nodeValue : "";
            var category = engineDom.documentElement.attributes.getNamedItem("Category").nodeValue;

            // put hosts to hosts global array
            var hostNodes = engineDom.documentElement.getElementsByTagName("Host");
            for (var k=0; k < hostNodes.length; k++) {
               groowe_Hosts[hostNodes.item(k).textContent] = engineName;
            }

            var homepagesNodes = engineDom.documentElement.getElementsByTagName("Homepage");
            for (var k=0; k < homepagesNodes.length; k++) {
               srcUrl = homepagesNodes.item(k).attributes.getNamedItem("Src").nodeValue;
               targetUrl = homepagesNodes.item(k).attributes.getNamedItem("Target").nodeValue;
               groowe_Homepages[srcUrl] = targetUrl;
            }

            // put search engine in its category array
            groowe_SearchEngines[engineName] = iconName + "||" + category;
            if (groowe_Categories[category] == null) {
               groowe_Categories[category] = engineName;
            }
            else {
               groowe_Categories[category] += "||" + engineName;
            }
         }
         catch (e) {
            Groowe_Error(e);
         }
      }
      groowe_SearchEngines.sort();
   }
   catch (e) {
      Groowe_Error(e);
   }
}



//
// Populate history drop down with items from preferences
//
function Groowe_PopulateHistory()
{
   try {

      // clear drop down
      var menu = document.getElementById("Groowe-SearchBoxHistory");
      for(var i=menu.childNodes.length - 1; i>=0; i--) {
         menu.removeChild(menu.childNodes.item(i));
      }

      // load history from preferences
      var history = "";
      branch = groowe_prefs.getBranch("groowebar.");
      try {history = branch.getCharPref("History");} catch (e) {}
      terms = history.split("||");

      //groowe_history.clear();

      // put items to drop down
      for(var i=0; i<terms.length; i++) {
         //var tempItem = null;
         tempItem = document.createElement("menuitem");
         tempItem.setAttribute("label", decodeURIComponent(terms[i]));
         groowe_history.push(terms[i]);
         menu.appendChild(tempItem);
      }
   }
   catch (e) {
      Groowe_Error(e);
   }
}


function ClearElements(o)
{
   for (var i=o.childNodes.length - 1; i>=0; i--) {
      ClearElements(o.childNodes.item(i));
      o.removeChild(o.childNodes.item(i));
   }
}

//
// Populate search engines drop down menu
//
function Groowe_ReadEngines()
{
   try {
      var menuBtn = document.getElementById("Groowe-SearchEnginesBtn");

      ClearElements(menuBtn);

      var menu = document.createElement("menupopup");

      // add favorites engines to popup menu root
      for (var key in groowe_Favorites) {
         var fav = groowe_SearchEngines[groowe_Favorites[key]];
         if (fav != null) {
            var tmp = fav.split("||");
            var tempItem = null;
            tempItem = document.createElement("menuitem");
            tempItem.setAttribute("label", groowe_Favorites[key]);
            tempItem.setAttribute("oncommand", "Groowe_LoadSearchEngine(\"" + groowe_Favorites[key] + "\");");
            tempItem.setAttribute("tooltiptext", groowe_Favorites[key]);
            tempItem.setAttribute("class", "menuitem-iconic");
            if ((tmp[0].length > 7) && (tmp[0].indexOf("://") != -1)) {
               tempItem.setAttribute("image", tmp[0]);
               tempItem.setAttribute("validate", "never");  // load from cache if possible
            }
            else {
               tempItem.setAttribute("image", "chrome://groowebar/skin/icons/" + tmp[0] + ".png");
            }
            menu.appendChild(tempItem);
         }
      }

      // separator
      menu.appendChild(document.createElement("menuseparator"));

      // adding categories sub-menus
      for (var category in groowe_Categories) {
         try
         {
            var tempMenu = null;
            tempMenu = document.createElement("menu");
            tempMenu.setAttribute("label", category);
            tempMenu.setAttribute("tooltiptext", category);

            var tempPopupMenu = null;
            tempPopupMenu = document.createElement("menupopup");

            var categoryEngines = groowe_Categories[category].toString().split("||");
            categoryEngines.sort();

            for (var i = 0; i < categoryEngines.length; i++) {
               var tempItem = null;
               tempItem = document.createElement("menuitem");
               tempItem.setAttribute("label", categoryEngines[i]);
               tempItem.setAttribute("class", "menuitem-iconic");
               tempItem.setAttribute("oncommand", "Groowe_LoadSearchEngine(\"" + categoryEngines[i] + "\");");
               var icon = groowe_SearchEngines[categoryEngines[i]].toString().split("||")[0];
               if ((icon.length > 7) && (icon.indexOf("://") != -1)) {
                  tempItem.setAttribute("image", icon);
                  tempItem.setAttribute("validate", "never");  // load from cache if possible
               }
               else {
                  tempItem.setAttribute("image", "chrome://groowebar/skin/icons/" + icon + ".png");
               }
               tempPopupMenu.appendChild(tempItem);
            }
            tempMenu.appendChild(tempPopupMenu);
            menu.appendChild(tempMenu);
         }
         catch (e)
         {
            Groowe_Error(e);
         }
      }


      var tempItem = null;
      tempItem = document.createElement("menuitem");
      tempItem.setAttribute("label", "Add more toolbars...");
      tempItem.setAttribute("oncommand", "Groowe_GoToURL('http://groowe.com/firefox_toolbar_plugins.php?ver=" + groowe_version + "')");
      tempItem.setAttribute("tooltiptext", "Add even more search engines and sites to Groowe Search Toolbar");
      tempItem.setAttribute("class", "menuitem-iconic");
      tempItem.setAttribute("image", "chrome://groowebar/skin/icons/Star.png");
      menu.appendChild(tempItem);


      // separator
      menu.appendChild(document.createElement("menuseparator"));

      // additional menu items
      var tempItem = null;
      tempItem = document.createElement("menuitem");
      tempItem.setAttribute("label", "Customization & Options");
      tempItem.setAttribute("oncommand", "Groowe_Options()");
      tempItem.setAttribute("tooltiptext", "Customize Groowe Search Toolbar");
      tempItem.setAttribute("class", "menuitem-iconic");
      tempItem.setAttribute("image", "chrome://groowebar/skin/icons/Tools.png");
      menu.appendChild(tempItem);

      menuBtn.appendChild(menu);
   }
   catch(e) {
      Groowe_Error(e);
   }
}



//
// Load search enigne
// Read XML config file and create toolbar for selected search engine
//
function Groowe_LoadSearchEngine(searchEngineName)
{
   if ((!groowe_forceEngineReload) && (searchEngineName == groowe_currentEngine)) {
      return;
   }
   groowe_forceEngineReload = false;

   var xmlContent;
   try {
      xmlContent = Groowe_GetPluginContent(searchEngineName);
      if (xmlContent == "") {
         xmlContent = Groowe_GetContents("chrome://groowebar/content/data/engines/" + searchEngineName + ".xml");
      }
      var parser = new DOMParser();
      var dom = parser.parseFromString(xmlContent, "text/xml");
      var logoName = dom.documentElement.getElementsByTagName("Logo").item(0).attributes.getNamedItem("LogoID").nodeValue;
      var btnSearchEngines = document.getElementById("Groowe-SearchEnginesBtn");

      if ((logoName.length > 7) && (logoName.indexOf("://") != -1)) {
         btnSearchEngines.setAttribute("image", logoName);
         btnSearchEngines.setAttribute("validate", "never");  // load from cache if possible
      }
      else {
         btnSearchEngines.setAttribute("image", "chrome://groowebar/skin/logos/" + logoName + ".png");
      }

      // delete existing buttons from container
      var container = document.getElementById("Groowe-ButtonContainer");
      for (var i = container.childNodes.length; i > 0; i--) {
         container.removeChild(container.childNodes[0]);
      }

      // load current engine search keywords field
      if (dom.documentElement.getElementsByTagName("KeywordsFieldName").length != 0) {
         groowe_currentQueryField = dom.documentElement.getElementsByTagName("KeywordsFieldName").item(0).textContent;
      }
      else {
         groowe_currentQueryField = "";
      }

      // load hosts for current site
      groowe_currentHosts = new Array();
      var hostNodes = dom.documentElement.getElementsByTagName("Host");
      for (var i = 0; i < hostNodes.length; i++) {
         groowe_currentHosts.push(hostNodes.item(i).textContent);
      }

      var nodeList = dom.documentElement.getElementsByTagName("Items");
      var itemsNode = nodeList.item(0);
      var order = "";

      // read native order
      for (var i = 0; i < itemsNode.childNodes.length; i++) {
         var node = itemsNode.childNodes.item(i);
         if (node.nodeName == "Item") {
            if (node.attributes.getNamedItem("ID") != null) {
               var id = node.attributes.getNamedItem("ID").nodeValue;
               order += (order != "") ? ";" + id : id;
            }
         }
      }

      // read order xml element
      if (dom.documentElement.getElementsByTagName("Order").length != 0) {
         var tmp = dom.documentElement.getElementsByTagName("Order").item(0).textContent;
         if (tmp != "") {
            order = tmp;
         }
      }

      // read customized order setting
      var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
      var branch = prefs.getBranch("groowebar.order.");
      try{ order = branch.getCharPref(searchEngineName);} catch(e) {}

      groowe_defaultURL = null;

      // add buttons by order
      var orderArray = order.split(";");
      for (var j = 0; j < orderArray.length; j++) {
         var id = orderArray[j];

         if (id == 0) {
            container.appendChild(document.createElement("toolbarseparator"));
         }
         else {
            for (var i = 0; i < itemsNode.childNodes.length; i++) {
               var node = itemsNode.childNodes.item(i);
               if ((node.nodeName == "Item") && (node.attributes.getNamedItem("ID") != null) && (node.attributes.getNamedItem("ID").nodeValue == id)) {
                  var caption = (groowe_showCaptions) ? node.attributes.getNamedItem("Caption").nodeValue : "";
                  var tooltip = (node.attributes.getNamedItem("Hint") != null) ? node.attributes.getNamedItem("Hint").nodeValue + " (Hold CTRL for new tab)" : "";
                  var icon = (node.attributes.getNamedItem("IconID") != null) ? node.attributes.getNamedItem("IconID").nodeValue : null;

                  var url = node.attributes.getNamedItem("URL").nodeValue;
                  var isMenu = (node.attributes.getNamedItem("IsMenu") != null) ? true : null;


                  if (groowe_defaultURL == null) {
                     groowe_defaultURL = url;
                  }

                  var tempButton = null;
                  tempButton = document.createElement("toolbarbutton");
                  if (node.hasChildNodes()) {
                     tempButton.setAttribute("type", (isMenu != null) ? "menu" : "menu-button");
                     if ((groowe_user_agent.indexOf("Firefox/") != -1) || (groowe_browser == "Firefox")) {
                        tempButton.setAttribute("class", "ffGroowe-Menu-Button");
                     }
                     else {
                        tempButton.setAttribute("class", "mzGroowe-Menu-Button");
                        tempButton.setAttribute("orient", "horizontal");
                     }
                  }
                  tempButton.setAttribute("label", caption);
                  tempButton.setAttribute("tooltiptext", tooltip);
                  tempButton.setAttribute("oncommand", "Groowe_Search(event, \"" + url + "\")");
                  if (icon != null) {
                     if ((icon.length > 7) && (icon.indexOf("://") != -1)) {
                        tempButton.setAttribute("image", icon);
                        tempButton.setAttribute("validate", "never");  // load from cache if possible
                     }
                     else {
                        tempButton.setAttribute("image", "chrome://groowebar/skin/icons/" + icon + ".png");
                     }
                  }
                  if (node.hasChildNodes()) {
                     var tempMenu = null;
                     tempMenu = document.createElement("menupopup");
                     Groowe_AddMenuItems(tempMenu, node);
                     tempButton.appendChild(tempMenu);
                  }
                  container.appendChild(tempButton);
               }
            }
         }
      }
      groowe_currentEngine = searchEngineName;
   }
   catch(e) {
      Groowe_Error(e);
   }
}


//
// Add menu items in for drop down buttons in LoadSearchToolbar
//
function Groowe_AddMenuItems(parentObj, node)
{
   for (var i = 0; i < node.childNodes.length; i++) {
      var itemNode = node.childNodes.item(i);
      if (itemNode.nodeName == "Item") {
         // For each search term, create a menu item element

         var caption = itemNode.attributes.getNamedItem("Caption").nodeValue;
         var tooltip = (itemNode.attributes.getNamedItem("Hint") != null) ? itemNode.attributes.getNamedItem("Hint").nodeValue + " (Hold CTRL for new tab)" : "";
         var url = (itemNode.attributes.getNamedItem("URL") != null) ? itemNode.attributes.getNamedItem("URL").nodeValue : "";
         var icon = (itemNode.attributes.getNamedItem("IconID") != null) ? itemNode.attributes.getNamedItem("IconID").nodeValue : null;

         if (itemNode.hasChildNodes()) {
            var tempMenu = null;
            tempMenu = document.createElement("menu");
            tempMenu.setAttribute("label", caption);
            tempMenu.setAttribute("tooltiptext", tooltip);
            if (icon != null) {
               tempMenu.setAttribute("class", "menuitem-iconic");
               tempMenu.setAttribute("image", "chrome://groowebar/skin/icons/" + icon);
            }
            var tempMenuPopup = null
            tempMenuPopup = document.createElement("menupopup");

            Groowe_AddMenuItems(tempMenuPopup, itemNode);

            tempMenu.appendChild(tempMenuPopup);
            parentObj.appendChild(tempMenu);
         }
         else {
            var tempItem = null;
            if (caption == "-") {
               tempItem = document.createElement("menuseparator");
            }
            else {
               tempItem = document.createElement("menuitem");
               tempItem.setAttribute("label", caption);
               tempItem.setAttribute("tooltiptext", tooltip);
               if (icon != null) {
                  tempItem.setAttribute("class", "menuitem-iconic");
                  if (icon != null) {
                     if ((icon.length > 7) && (icon.indexOf("://") != -1)) {
                        tempItem.setAttribute("image", icon);
                        tempItem.setAttribute("validate", "never");  // load from cache if possible
                     }
                     else {
                        tempItem.setAttribute("image", "chrome://groowebar/skin/icons/" + icon + ".png");
                     }
                  }
               }
               tempItem.setAttribute("oncommand", "Groowe_Search(event, \"" + url + "\"); event.stopPropagation();");
            }
            parentObj.appendChild(tempItem);
         }
      }
   }
}



//
// Navigate to specified URL
//
function Groowe_GoToURL(URL)
{
   if (groowe_newTab) {
      const newTab = window.getBrowser().addTab(URL);
      window.getBrowser().selectedTab = newTab;
   }
   else {
      window._content.document.location = URL;
      window.content.focus();
   }
}


//
// Navigate to specified URL in new tabe
//
function Groowe_NewTabURL(url)
{
   const newTab = window.getBrowser().addTab(url);
   //window.getBrowser().selectedTab = newTab;
}


//
// Search for keywords endered in the search box with specified URL
//
function Groowe_Search(event, url)
{
   if (groowe_first_search) {
      groowe_first_search = false;
      var xmlContent = Groowe_AutoCheckUpdates();
   }

   url = Groowe_PrepareURL(url);
   if (event.ctrlKey) {
      Groowe_NewTabURL(url);
   }
   else {
      Groowe_GoToURL(url);
   }
}


function Groowe_AutoCheckUpdates()
{
    var url = "http://groowe.com/toolbar/ff_auto_check_updates.php?ver=" + groowe_version;


    try {
        var appInfo=Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
        var isOnBranch = appInfo.platformVersion.indexOf("1.8") == 0;
        var ios=Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
        var uri = ios.newURI(url, null, null);
        var channel=ios.newChannelFromURI(uri);
        var observer={
            onStreamComplete:function(aLoader, aContext, aStatus, aLength, aResult)
            {
                var parser = new DOMParser();
                var dom = parser.parseFromBuffer(aResult, aLength, "text/xml");

                if (dom.documentElement.attributes.getNamedItem("Display").nodeValue != "false") {
                   var title = "";
                   var text = "";
                   var url = "";

                   // read messages and urls from xml
                   if (dom.documentElement.getElementsByTagName("Title").length != 0) {
                      title = dom.documentElement.getElementsByTagName("Title").item(0).textContent;
                   }
                   if (dom.documentElement.getElementsByTagName("Text").length != 0) {
                      text = dom.documentElement.getElementsByTagName("Text").item(0).textContent;
                   }
                   if (dom.documentElement.getElementsByTagName("URL").length != 0) {
                      url = dom.documentElement.getElementsByTagName("URL").item(0).textContent;
                   }

                   var groowe_upgrade_btn = document.getElementById("Groowe-Upgrade");
                   groowe_upgrade_btn.setAttribute("oncommand", "Groowe_GoToURL('" + url + "')");
                   groowe_upgrade_btn.setAttribute("tooltiptext", text);
                   groowe_upgrade_btn.hidden = false;
                }
            }
        };

        var sl=Components.classes["@mozilla.org/network/stream-loader;1"].createInstance(Components.interfaces.nsIStreamLoader);
        if (isOnBranch)
        {
            sl.init(channel, observer, null);
        }
        else {
            sl.init(observer);
            channel.asyncOpen(sl, channel);
        }
   }
   catch (e) {
      Groowe_Error(e);
   }
}

//
// Switch to next search engine in favorites list and execute default search
//
function Groowe_SearchNext(event)
{
   var index = 0;

   for (var i=0; i < groowe_Favorites.length; i++) {
      if (groowe_Favorites[i] == groowe_currentEngine) {
         index = (i+1 == groowe_Favorites.length) ? 0 : i+1;
         break;
      }
   }
   Groowe_LoadSearchEngine(groowe_Favorites[index]);

   if (event) {
      Groowe_DefaultSearch(event.ctrlKey);
   }
   else {
      Groowe_DefaultSearch(false);
   }
}

function Groowe_NextEngine()
{
   var index = 0;

   for (var i=0; i < groowe_Favorites.length; i++) {
      if (groowe_Favorites[i] == groowe_currentEngine) {
         index = (i+1 == groowe_Favorites.length) ? 0 : i+1;
         break;
      }
   }
   Groowe_LoadSearchEngine(groowe_Favorites[index]);
   Groowe_SetFocus(null);
}

function Groowe_PrevEngine()
{
   var index = groowe_Favorites.length - 1;

   for (var i=index; i >= 0; i--) {
      if (groowe_Favorites[i] == groowe_currentEngine) {
         index = (i-1 == -1) ? groowe_Favorites.length - 1 : i-1;
         break;
      }
   }
   Groowe_LoadSearchEngine(groowe_Favorites[index]);
   Groowe_SetFocus(null);
}


function Groowe_SearchBoxKeyPress(event)
{
   if (event.which == 13) {
      Groowe_DefaultSearch(event.ctrlKey);
   }
}

function Groowe_SetFocus(event)
{
   var searchBox = document.getElementById("Groowe-SearchBox");
   searchBox.select();
   searchBox.focus();
}


//
// Execute default search (first button in toolbar action)
//
function Groowe_DefaultSearch(toNewTab)
{
   try {
      var url = Groowe_PrepareURL(groowe_defaultURL);
      if (toNewTab) {
         Groowe_NewTabURL(url);
      }
      else {
         Groowe_GoToURL(url);
      }
   }
   catch (e) {
      Groowe_Error(e);
   }
}



//
// Prepare URL, replace all variables with encoded values
//
function Groowe_PrepareURL(url)
{
   try {
      var searchBox = document.getElementById("Groowe-SearchBox");
      var query = Groowe_TrimString(searchBox.value);

      Groowe_PutToHistory(query);

      var preparedUrl = new String(url)
      preparedUrl = preparedUrl.replace("$utf8query", encodeURIComponent(query));
      try {
         preparedUrl = preparedUrl.replace("$title", encodeURIComponent(window._content.document.title));
         preparedUrl = preparedUrl.replace("$host", encodeURIComponent(window._content.document.location.hostname));
         preparedUrl = preparedUrl.replace("$url", encodeURIComponent(window._content.document.location.href));
      }
      catch (e) {
         Groowe_Error(e);
      }

      url = preparedUrl.replace("$query", escape(query.replace(" ",  "+")));
   }
   catch(e) {
      Groowe_Error(e);
   }

   return url;
}


//
// Put search terms to history and save history to preferences
//
function Groowe_PutToHistory(query)
{
   var menu = document.getElementById("Groowe-SearchBoxHistory");
   groowe_find_in_page_term = "";   // clear last search terms


   // if exists, remove from history, we will put it back on the top
   for (var i = 0; i < menu.childNodes.length; i++) {
      var node = menu.childNodes.item(i);
      if (node.label == query) {
         menu.removeChild(menu.childNodes.item(i));
      }
   }

   // put search keywords on the top of drop down list
   var tempItem = null;
   tempItem = document.createElement("menuitem");
   tempItem.setAttribute("label", query);
   menu.insertBefore(tempItem, menu.firstChild);

   // save history to pref, limit to number of entries
   var historyLimit = 40;
   var history = "";
   for(var i=0; ((i<menu.childNodes.length) && (i < historyLimit)); i++) {
      if (history != "") {
         history += "||";
      }
      history += encodeURIComponent(menu.childNodes.item(i).attributes.getNamedItem("label").nodeValue);
//      history += menu.childNodes.item(i).label;
   }
   branch = groowe_prefs.getBranch("groowebar.");
   branch.setCharPref("History", history);
}



//
// Clear current drop down history, and clear saved history as well
//
function Groowe_ClearSearchHistory()
{
      var menu = document.getElementById("Groowe-SearchBoxHistory");
      for (var i = menu.childNodes.length-1; i >= 0; i--) {
         menu.removeChild(menu.childNodes.item(i));
      }
      branch = groowe_prefs.getBranch("groowebar.");
      branch.setCharPref("History", "");

      var searchBox = document.getElementById("Groowe-SearchBox");
      searchBox.value = "";
}




//
// Open Groowe options dialog
//
function Groowe_Options()
{
   try {
      var branch = groowe_prefs.getBranch("groowebar.");
      branch.setBoolPref("IsChanged", false);

      window.openDialog("chrome://groowebar/content/options.xul", "Groowe Search Toolbar Options", "centerscreen,chrome,modal", groowe_SearchEngines, groowe_Favorites);

      // if preferences changed, initialize toolbar with new settings
      groowe_refresh = branch.getBoolPref("IsChanged");
      Groowe_Init();
   }
   catch(e) {
      Groowe_Error(e);
   }
}

function Groowe_CheckOnDropDown()
{
   if (groowe_refresh) {
      groowe_refresh = false;
      Groowe_Init();
   }
}




//
// Open Groowe about dialog
//
function Groowe_About()
{
   window.openDialog("chrome://groowebar/content/about.xul", "About...", "centerscreen,chrome,modal,resizable", groowe_version);
}

//
// Groowe info window
//
function Groowe_Alert(title, msg, type)
{
   window.openDialog("chrome://groowebar/content/alert.xul", "Groowebar : " + title, "centerscreen,chrome,modal,resizable", title, msg, type);
   return groowe_alert_result;
}

//
//
//
function Groowe_SetAlertResult(val)
{
   groowe_alert_result = val;
}



//
// Event listener
//
var Groowe_Listener =
{
   QueryInterface: function(aIID)
   {
      if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
      aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
      aIID.equals(Components.interfaces.nsISupports))
      return this;
      throw Components.results.NS_NOINTERFACE;
   },

   onProgressChange: function (aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress)
   {
   },

   onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage)
   {
   },

   onSecurityChange: function(aWebProgress, aRequest, aState) {},

   onLinkIconAvailable: function(a) {},

   onStateChange: function (aWebProgress, aRequest, aStateFlags, aStatus)
   {
      try {
         if ((aStateFlags & (Components.interfaces.nsIWebProgressListener.STATE_START)) && (aRequest.URI != null)) {
            var targetURL = aRequest.URI.spec;

            // if autoselect is on and host recognized, switch to search engine
            if (groowe_autoSelect) {
               for (var grooweHome in groowe_Homepages) {
                  if (grooweHome == targetURL) {
                     Groowe_GoToURL(groowe_Homepages[grooweHome]);
                     return;
                  }
               }
            }
         }
      }
      catch(e) {
         Groowe_Error(e);
      }

      // highlight terms after page load if Highlight buttons is checked
      if(aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_STOP)
      {
         var hbutton = document.getElementById("Groowe-HighlightBtn");
         if(hbutton.checked)
         {
            Groowe_AddHighlighting();
         }
      }
   },


   onLocationChange: function(aProgress, aRequest, aLocation)
   {
      if (aLocation) {
         // insert search terms from search engine to search box
         var keywords = "";
         var url = aLocation.spec;

         // extract target host
         var host = "";
         if (/^http:\/\/([^\/]*)/i.test(url)) {
            host = RegExp.$1;
            if (/^www\.(.*)/i.test(host)) {
               host = RegExp.$1;
            }
         }

         // add highlighting
         var hbutton = document.getElementById("Groowe-HighlightBtn");
         if(hbutton.checked == true) {
            Groowe_AddHighlighting();
         }

         // if autoselect is on and host recognized, switch to search engine
         if (groowe_autoSelect) {
            var isSwitched = false;



            for (var grooweHost in groowe_Hosts) {
               if ((grooweHost == host) || ((grooweHost.indexOf("*") == 0) && (host.indexOf(grooweHost.substr(1)) > -1))) {
               //if (grooweHost == host) {
                  if (groowe_prevEngine == "") {
                     // remember prev engine only if it is empty
                     groowe_prevEngine = groowe_currentEngine;
                  }
                  Groowe_LoadSearchEngine(groowe_Hosts[grooweHost]);
                  isSwitched = true;
                  break;
               }
            }

            if ((!isSwitched) && (groowe_prevEngine != "")) {
               // if engine on autoselect not found, and prevEngine exists
               // switch back to prev engine
               Groowe_LoadSearchEngine(groowe_prevEngine);
               groowe_prevEngine = "";
            }
         }

         // read search terms only if we are searching currently selected engine
         var readSearchTerms = false;
         for (var index in groowe_currentHosts) {
            if (groowe_currentHosts[index] == host) {
               readSearchTerms = true;
               break;
            }
         }

         // read search terms from URL and put them in serach box
         var uri = url.split("?")[1];
         if ((uri != undefined) && (groowe_currentQueryField != "") && (readSearchTerms)) {
            uri = "?" + uri;
            var q1 = "?" + groowe_currentQueryField + "=";
            var q2 = "&" + groowe_currentQueryField + "=";
            for (var i = 0; i < uri.length-(q1.length-1); i++) {
               var queryBegin = "";
               for (var k=0; k<q1.length; k++) {
                  queryBegin += uri[i+k];
               }
               if ((queryBegin == q1) || (queryBegin == q2)) {
                  for (var j=i+q1.length; j < uri.length; j++) {
                     if (uri[j] == "&") {
                        break;
                     }
                     else {
                        keywords += uri[j];
                     }
                  }
                  break;
               }
            }
         }

         // if keywords fourn in URL, put them to search box
         if (keywords != "") {
            keywords = decodeURIComponent(keywords.replace(/\+/g, " "));
            document.getElementById("Groowe-SearchBox").value = keywords;
            Groowe_PutToHistory(keywords);
         }
      }
   }
}

//
// Hide/Unhide
//
function Groowe_ToggleShowHide()
{
   try {
      var grooweToolbar = document.getElementById("Groowe-Toolbar");
      grooweToolbar.collapsed = !grooweToolbar.collapsed;
   }
   catch (e) {
      Groowe_Error(e);
   }
}


///////////////////// BEGIN HIGHLIGHT FUNCTIONS //////////////////////////

//
// Toggle highlight on/off
//
function Groowe_HighlightOnOff()
{
   var btnHighlight = document.getElementById("Groowe-HighlightBtn");
   if(btnHighlight.checked == true)
   {
      btnHighlight.checked = false;
      //Groowe_HighLightInPage("off");
      Groowe_RemoveHighlighting();
   }
   else
   {
      btnHighlight.checked = true;
      //Groowe_HighLightInPage("on");
      Groowe_AddHighlighting();
   }
}


function Groowe_AddHighlighting(win)
{
   try {
      if(!win) {
         win = window._content;
      }

      Groowe_RemoveHighlighting(win);

      var i=0; // Avoid multiple redeclaration warnings

      for (i=0; win.frames && i < win.frames.length; i++)
      {
         Groowe_AddHighlighting(win.frames[i]);
      }

      var doc = win.document;
      if (!doc) {
         return;
      }
      var body = doc.body;
      if (!body) {
         return;
      }

      var count = body.childNodes.length;

      var terms = Groowe_TrimString(document.getElementById("Groowe-SearchBox").value);
      var termsArray = Groowe_SplitKeywords(terms);

      for(i = 0; i < termsArray.length; i++) {
         var term = termsArray[i];

         var span = doc.createElement("span");
         span.setAttribute("style", "background-color: " + Groowe_highlightColors[i % Groowe_highlightColors.length]);
         span.setAttribute("id", "groowe-highlight");

         var searchRange = doc.createRange();
         searchRange.setStart(body,  0);
         searchRange.setEnd(body, count);

         var start = doc.createRange();
         start.setStart(body, 0);
         start.setEnd(body, 0);

         var end = doc.createRange();
         end.setStart(body, count);
         end.setEnd(body, count);

         var result = null;
         while ((result = groowe_finder.Find(term, searchRange, start, end))) {
            var surround = span.cloneNode(true);

            var startContainer = result.startContainer;
            var startOffset = result.startOffset;
            var endOffset = result.endOffset;

            var docfrag = result.extractContents();
            var before = startContainer.splitText(startOffset);
            var parent = before.parentNode;

            //alert(parent.nodeName);
//            alert(parent.parentNode.nodeName);
            if ((parent.parentNode != undefined) && (parent.parentNode.nodeName.toLowerCase() == "input")) {
               parent.insertBefore(docfrag, before);
               surround = parent;
            }
            else {
               surround.appendChild(docfrag);
               parent.insertBefore(surround, before);
            }
            start = surround.ownerDocument.createRange();
            start.setStart(surround, surround.childNodes.length);
            start.setEnd(surround, surround.childNodes.length);
         }
      }
   }
   catch (e) {
      Groowe_Error(e);
   }
}


function Groowe_RemoveHighlighting(win)
{
   // Remove highlighting.  We use the find API again rather than
   // searching for our span elements by id so that we gain access to the
   // anonymous content that nsIFind searches.

   try {
      if(!win)
           win = window.self;

      var i=0; // Avoid multiple redeclaration warnings

      for(i=0; win.frames && i < win.frames.length; i++)
      {
         Groowe_RemoveHighlighting(win.frames[i]);
      }

      var doc = win.document;
      if(!doc) { return; }
      var body = doc.body;
      if(!body) { return; }

      var count = body.childNodes.length;
      var terms = Groowe_TrimString(document.getElementById("Groowe-SearchBox").value);
      var termsArray = Groowe_SplitKeywords(terms);

      for(i=0; i<termsArray.length; i++)
      {
         var term = termsArray[i];

         var searchRange = doc.createRange();
         searchRange.setStart(body, 0);
         searchRange.setEnd(body, count);

         var startPt = doc.createRange();
         startPt.setStart(body, 0);
         startPt.setEnd(body, 0);

         var endPt = doc.createRange();
         endPt.setStart(body, count);
         endPt.setEnd(body, count);

         var retRange = null;
         while((retRange = groowe_finder.Find(term, searchRange, startPt, endPt)))
         {
            var startContainer = retRange.startContainer;
            var elem = null;
            try { elem = startContainer.parentNode; } catch (e) { }
            if (elem)
            {
               // Avoid multiple redeclaration warnings
               var docfrag = null;
               var child = null;
               var next = null;
               var parent = null;

               if (elem.getAttribute("id") == "groowe-highlight")
               {
                  docfrag = doc.createDocumentFragment();
                  next = elem.nextSibling;
                  parent = elem.parentNode;

                  while((child = elem.firstChild)) {
                     docfrag.appendChild(child);
                  }

                  startPt = doc.createRange();
                  startPt.setStartAfter(elem);

                  parent.removeChild(elem);
                  parent.insertBefore(docfrag, next);
               }
               else
               {
                  // For whatever reason, we did not highlight this instance (or it
                  // appears that we did not). Let's walk up the DOM tree to see if
                  // we can find a span higher up. This fixes a bug in GBL with
                  // poorly coded HTML.
                  try {
                     while((elem = elem.parentNode))
                     {
                        if(elem.getAttribute("id") == "groowe-highlight")
                        {
                           docfrag = doc.createDocumentFragment();
                           next = elem.nextSibling;
                           parent = elem.parentNode;

                           while((child = elem.firstChild)) {
                              docfrag.appendChild(child);
                           }

                           startPt = doc.createRange();
                           startPt.setStartAfter(elem);

                           parent.removeChild(elem);
                           parent.insertBefore(docfrag, next);
                           break;
                        }
                     }
                  } catch(e) {}
                  startPt = doc.createRange();
                  startPt.setStart(retRange.endContainer, retRange.endOffset);
               }
            }
            else
            {
               startPt = doc.createRange();
               startPt.setStart(retRange.endContainer, retRange.endOffset);
            }

            startPt.collapse(true);
         }
      }
   }
   catch (e) {
      Groowe_Error(e);
   }
}

///////////////////////// END HIGHLIGHT FUNCTIONS //////////////////////////





//
// Put search keywords in find on page dropdown menu
//
function Groowe_PopulateFindOnPage()
{
   try {
      var searchBox = document.getElementById("Groowe-SearchBox");
      var term = searchBox.value;
      //alert(term);
      var terms = Groowe_SplitKeywords(term);

      var menu = document.getElementById("Groowe-FindOnPageMenu");
      //var menuBtn = document.getElementById("Groowe-FindOnPageBtn");
      for (var i = menu.childNodes.length - 1; i>=0; i--) {
         menu.removeChild(menu.childNodes.item(i));
      }
      //createElement("menupopup");

      if (terms.length != 0) {
         for (var i in terms) {
            var tempItem = null;
            tempItem = document.createElement("menuitem");
            tempItem.setAttribute("label", terms[i]);
            tempItem.setAttribute("oncommand", "Groowe_FindInPage(\"" + terms[i] + "\", event);");
            tempItem.setAttribute("tooltiptext", "Find next occurrence of '" + terms[i] + "' on this page");
            tempItem.setAttribute("class", "menuitem-iconic");
            tempItem.setAttribute("image", "chrome://groowebar/skin/icons/Search.png");
            menu.appendChild(tempItem);
         }
      }
      else {
         var tempItem = null;
         tempItem = document.createElement("menuitem");
         tempItem.setAttribute("label", "(Empty)");
         tempItem.setAttribute("tooltiptext", "No search keywords");
         menu.appendChild(tempItem);
      }
      //menuBtn.appendChild(menu);
   }
   catch (e) {
      Groowe_Error(e);
   }
}

function Groowe_SearchNextOccurrence(e)
{
   try {
      if (groowe_find_in_page_term == "") {
         var terms = Groowe_SplitKeywords(document.getElementById("Groowe-SearchBox").value);
         if (terms.length != 0) {
            groowe_find_in_page_term = terms[0];
         }
      }

      Groowe_FindInPage(groowe_find_in_page_term, e);
   }
   catch (e) {
      Groowe_Error(e);
   }
}

//
// Find and select (highlight) term on current page
//
function Groowe_FindInPage(term, e)
{
   if (term != "") {
      groowe_find_in_page_term = term;
      // This is shamelessly taken from Firebird findUtils.js
      var found = false;
      var focusedWindow = document.commandDispatcher.focusedWindow;
      if (!focusedWindow || focusedWindow == window)
         focusedWindow = window._content;
      var findInst = getBrowser().webBrowserFind;
      var findInFrames = findInst.QueryInterface(Components.interfaces.nsIWebBrowserFindInFrames);
      findInFrames.rootSearchFrame = window._content;
      findInFrames.currentSearchFrame = focusedWindow;

      // setup the find instance
      findInst.searchString = term;

      if(e)
      {
         // we hardcode the search options here with what seems to make most sense
         findInst.searchFrames   = true;
         findInst.wrapFind    = true;
         findInst.entireWord     = e.altKey;
         findInst.findBackwards  = e.shiftKey;
         findInst.matchCase      = e.ctrlKey;

         if(findInst.entireWord)
            findInst.searchString  = ' ' + str + ' ';
      }
      found = findInst.findNext();
   }
}


//
// Trim spaces on both sides, replace more then one space with one
//
function Groowe_TrimString(string)
{
   if (!string) return "";

   var value = string.replace(/^\s+/, '');
   value = string.replace(/\s+$/, '');
   value = value.replace(/\s+/g, ' ');
   return value;
}



//
// Nicely split keywords, by Googlebarlite
//
function Groowe_SplitKeywords(searchwords)
{
   var string = Groowe_TrimString(searchwords);
   var buffer = "";
   var inQuotes = false;
   var inWord = false;
   var myArray = new Array();

   for(var i=0; i<string.length; i++) {
      var c = string.charAt(i);
      switch(c) {
      case ' ':
         if(inQuotes == false) {
            myArray.push(buffer);
            buffer = "";
            if(inWord == true)
               inWord = false;
         }
         else {
            buffer += c;
         }
         break;
      case '"':
         if (inQuotes == false) {
            inQuotes = true;
            inWord = true;
         }
         else {
            inQuotes = false;
            inWord = false;
         }
         break;
      case ',':
         break; // Ignore commas (regardless of where they are)
      case '+':
      case '~':
         if (inWord == false) {
            break;
         } // Ignore the ~ and + symbols that precede any search words
      default:
         if (inWord == false) {
            inWord = true;
         }
         buffer += c;
      }
   }

   if (buffer != "") {
      myArray.push(buffer);
   }

   return myArray;
}


function Groowe_GetDefaultURL(searchEngineName)
{

   var xmlContent;
   try {
      xmlContent = Groowe_GetContents("chrome://groowebar/content/data/engines/" + searchEngineName + ".xml");
      if (xmlContent == "") {
         xmlContent = Groowe_GetPluginContent(searchEngineName);
      }
      var parser = new DOMParser();
      var dom = parser.parseFromString(xmlContent, "text/xml");

      var nodeList = dom.documentElement.getElementsByTagName("Items");
      var itemsNode = nodeList.item(0);
      var order = "";

      // read native order
      for (var i = 0; i < itemsNode.childNodes.length; i++) {
         var node = itemsNode.childNodes.item(i);
         if (node.nodeName == "Item") {
            if (node.attributes.getNamedItem("ID") != null) {
               var id = node.attributes.getNamedItem("ID").nodeValue;
               order += (order != "") ? ";" + id : id;
            }
         }
      }

      // read order xml element
      if (dom.documentElement.getElementsByTagName("Order").length != 0) {
         var tmp = dom.documentElement.getElementsByTagName("Order").item(0).textContent;
         if (tmp != "") {
            order = tmp;
         }
      }

      // read customized order setting
      var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
      var branch = prefs.getBranch("groowebar.order.");
      try{ order = branch.getCharPref(searchEngineName);} catch(e) {}

      groowe_defaultURL = null;

      // add buttons by order
      var orderArray = order.split(";");
      for (var j = 0; j < orderArray.length; j++) {
         var id = orderArray[j];

         if (id != 0) {
            for (var i = 0; i < itemsNode.childNodes.length; i++) {
               var node = itemsNode.childNodes.item(i);
               if ((node.nodeName == "Item") && (node.attributes.getNamedItem("ID") != null) && (node.attributes.getNamedItem("ID").nodeValue == id)) {
                  return node.attributes.getNamedItem("URL").nodeValue;
               }
            }
         }
      }
   }
   catch(e) {
       Groowe_Error(e);
   }
   return "";
}


function Groowe_MultiSearch(count)
{
   try {
      for (var i=0; ((i < groowe_Favorites.length) && (i < count)); i++) {
         var url = Groowe_GetDefaultURL(groowe_Favorites[i]);
         url = Groowe_PrepareURL(url);
         Groowe_NewTabURL(url);
      }
   }
   catch (e) {
      Groowe_Error(e);
   }
}



////////////////// BEGIN PLUGIN DOWNLOAD HANDLING FUNCTIONS //////////////////////////

//
//
//
function onTabLoad(aEvent) {
   var doc = aEvent.originalTarget;
   if (doc) {
      if (doc.location) {
          var targetURL = doc.location.href;
          var url = targetURL.split("?")[0];
          if ((url != undefined) && ( (url == "http://groowe.com/firefox-plugin-dl/") || (url == "http://www.groowe.com/firefox-plugin-dl/ "))) {
             pluginURL = Groowe_GetPluginURL(targetURL);
             if (pluginURL != "") {
                if (Groowe_Alert("Groowe Toolbar Plugin Download", "Plugin URL: " + pluginURL + "\nDownload this Groowe Toolbar plugin?" , "question")) {
                   Groowe_DownloadPlugin(pluginURL);
                }
                else {
                   doc.location = "http://groowe.com/firefox-plugin-dl/error.php?title=Canceled&msg=" + encodeURIComponent("Plugin installation has been canceled on your demand.");
                }
             }
             else {
                doc.location = "http://groowe.com/firefox-plugin-dl/error.php?title=Error&msg=" + encodeURIComponent("Plugin URL not specified.");
             }
          }
      }
   }
}


function Groowe_DownloadPlugin(url)
{
   try {
      var s = Groowe_GetContents(url);
      var fileName = Groowe_AnalyzePlugin(s);

      if (fileName != "") {

         // check is it in embedded engines
/*
         xmlContent = Groowe_GetContents("chrome://groowebar/content/data/engines.xml");
         var parser = new DOMParser();
         var dom = parser.parseFromString(xmlContent, "text/xml");
         var nodeList = dom.documentElement.getElementsByTagName("Engine");
         for (var i = 0; i < nodeList.length; i++) {
            if (nodeList.item(i).textContent == fileName) {
               Groowe_Alert("Error", "You can't overwrite plugins that are bundeled into Groowe Saerch Toolbar installationa.\nYou can only overwrite other Groowebar plugins.", "error");
               return;
            }
         }
*/
         /////


         var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
         file.append("GrooweFiles");
         if (!file.exists()) {
            file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0700);
         }

         file.append(fileName + ".xml");
         if (file.exists()) {
            if (!Groowe_Alert("Groowe Toolbar Plugin Exists", "'" + fileName + "' Groowe plugin already exists.\nOverwrite?", "question")) {
               Groowe_GoToURL("http://groowe.com/firefox-plugin-dl/error.php?title=Canceled&msg=" + encodeURIComponent("Plugin installation has been canceled on your demand."));
               return;
            }
         }

         var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
         converter.charset = "UTF-8";

         var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
         // use 0x02 | 0x10 to open file for appending.
         foStream.init(file, 0x02 | 0x08 | 0x20, 0664, 0); // write, create, truncate
         s = converter.ConvertFromUnicode(s);
         foStream.write(s, s.length);
         foStream.close();
         if (Groowe_Alert("Plugin successfully installed!", fileName + " successufully installed in category \"" + groowe_new_plugin_category +"\".\n\nAdd this plugin to your favorites list?", "question")) {
            if (!Groowe_InArray(groowe_Favorites, fileName)) {
               groowe_Favorites.push(fileName);
               var branch = groowe_prefs.getBranch("groowebar.");
               var favorites = groowe_Favorites.join("||");
               branch.setCharPref("Favorites", favorites);
            }
         }
         Groowe_Init();
         Groowe_GoToURL("http://groowe.com/firefox-plugin-dl/success.php?plugin=" + encodeURIComponent(fileName) + "&category=" + encodeURIComponent(groowe_new_plugin_category));
      }
   }
   catch (e) {
      Groowe_Error(e);
   }
}



function Groowe_AnalyzePlugin(s)
{
  var engineName = "";
  groowe_new_plugin_category = "";

   try {
      var parser = new DOMParser();
      var dom = parser.parseFromString(s, "text/xml");

      var logoName = dom.documentElement.getElementsByTagName("Logo").item(0).attributes.getNamedItem("LogoID").nodeValue;
      var engineName = dom.documentElement.attributes.getNamedItem("Name").nodeValue;
      groowe_new_plugin_category = dom.documentElement.attributes.getNamedItem("Category").nodeValue;

      var itemsNode = dom.documentElement.getElementsByTagName("Items").item(0);

      // read native order
      for (var i = 0; i < itemsNode.childNodes.length; i++) {
         var node = itemsNode.childNodes.item(i);
         if ((node.nodeName == "Item") && (node.attributes.getNamedItem("ID") != null)) {
            var caption = node.attributes.getNamedItem("Caption").nodeValue;
            var tooltip = (node.attributes.getNamedItem("Hint") != null) ? node.attributes.getNamedItem("Hint").nodeValue : "";
            var icon = (node.attributes.getNamedItem("IconID") != null) ? node.attributes.getNamedItem("IconID").nodeValue + '.png' : null;
            var url = node.attributes.getNamedItem("URL").nodeValue;
            var isMenu = (node.attributes.getNamedItem("IsMenu") != null) ? true : null;
         }
      }
      return engineName;
   }
   catch (e) {
      Groowe_Alert("Error", "Unable to intall!\nSyntax error in plugin file or plugin file doesn't exist.", "error");
      Groowe_GoToURL("http://groowe.com/firefox-plugin-dl/error.php?title=" + encodeURIComponent("Error, Unable to install") + "&msg=" + encodeURIComponent("Syntax error in plugin file or plugin file doesn't exist on specified URL."));
      return "";
   }
}

////////////////// END PLUGIN DOWNLOAD HANDLING FUNCTIONS //////////////////////////
